/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import Ubuntu.Components.Themes.Ambiance 0.1
import Ubuntu.Components.Popups 0.1
import "../common/ClockUtils.js" as Utils
import "../common"

// Page to edit or create a new alarm
Page {
    id: addAlarmPage

    // Property to determine if this is New/Edit mode
    property bool isNewAlarm

    // Property to store the index of the alarm to be edited
    property int alarmIndex

    visible: false
    flickable: null

    Component.onCompleted: {
        if (isNewAlarm)
            addAlarmPage.title = i18n.tr("Add Alarm")
        else {
            addAlarmPage.title = i18n.tr("Edit Alarm")
            readAlarm()
        }
    }

    Alarm {
        id: alarm
        onStatusChanged: {
            if (status != Alarm.Ready)
                return;
            if ((operation > Alarm.NoOperation) && (operation < Alarm.Reseting)) {
                reset();
                resetPage()
                pagestack.pop();
            }
        }
    }

    actions: [

    ]

    // Function to save a new alarm
    function saveNewAlarm() {
        var alarmTime = new Date();
        alarmTime.setHours(addAlarmFace.hours, addAlarmFace.minutes, 0)

        alarm.message = nameAlarm.text
        alarm.date = alarmTime
        alarm.enabled = true
        alarm.save()
    }

    // Function to read and show an existing alarm
    function readAlarm() {
        var temp_alarm = alarmModel.get(alarmIndex)

        alarm.message = temp_alarm.message
        alarm.type = temp_alarm.type
        alarm.daysOfWeek = temp_alarm.daysOfWeek
        alarm.enabled = temp_alarm.enabled
        alarm.date = temp_alarm.date

        addAlarmFace.hours = Qt.formatTime(alarm.date, "hh")
        addAlarmFace.hourHandRotation = addAlarmFace.hours * 15
        addAlarmFace.minutes = Qt.formatTime(alarm.date, "mm")
        addAlarmFace.minuteHandRotation = addAlarmFace.minutes * 6
    }

    function updateAlarm() {
        var temp_alarm = alarmModel.get(alarmIndex)

        var alarmTime = new Date()
        alarmTime.setHours(addAlarmFace.hours, addAlarmFace.minutes, 0)

        temp_alarm.message = nameAlarm.text
        temp_alarm.type = alarm.type
        temp_alarm.daysOfWeek = alarm.daysOfWeek
        temp_alarm.date = alarmTime
        temp_alarm.save()
        resetPage()
        pagestack.pop()
    }

    function resetPage() {
        addAlarmFace.hours = addAlarmFace.minutes = 0;
        addAlarmFace.hourHandRotation = addAlarmFace.minuteHandRotation = 0
    }

    Flickable {
        clip: true
        anchors.fill: parent
        contentHeight: mainColumn.height + units.gu(5)
        interactive: contentHeight > parent.height

        Column {
            id: mainColumn

            spacing: units.gu(1)
            anchors { left: parent.left; right: parent.right; top: parent.top; topMargin: units.gu(5)}

            AnalogFaceBase {
                id: addAlarmFace

                property int hours: 0
                property int minutes: 0
                property alias hourHandRotation: hourHand.rotationValue
                property alias minuteHandRotation: minuteHand.rotationValue

                onMinutesChanged: labelAlarm.text = Utils.zeroleft(addAlarmFace.hours, 2) + ":" + Utils.zeroleft(addAlarmFace.minutes, 2);
                onHoursChanged: labelAlarm.text = Utils.zeroleft(addAlarmFace.hours, 2) + ":" + Utils.zeroleft(addAlarmFace.minutes, 2)

                anchors.horizontalCenter: parent.horizontalCenter

                // Hour hand with touch/mouse drag support
                AnalogTouchHand {
                    id: hourHand
                    objectName: "hourHand"

                    onTimerValueChanged: addAlarmFace.hours = timerValue;

                    perAngle: 15;
                    totalValue: 24;
                    handHeight: units.gu(12.5); handWidth: units.gu(1);
                    z: minuteHand.z + 1;
                    animateFlag: false;
                    enabled: true;
                    grabMargin: units.gu(3);
                }

                // Minute hand with touch/mouse drag support
                AnalogTouchHand {
                    id: minuteHand

                    onTimerValueChanged: addAlarmFace.minutes = timerValue;

                    handHeight: units.gu(14.5); handWidth: units.gu(0.5);
                    animateFlag: false;
                    enabled: true;
                    grabMargin: units.gu(0)
                    grabHeight: units.gu(7)
                }

                // Label to show the current alarm time
                Item {
                    id: labelContainer;

                    z: parent.z + 2;
                    width: parent.innerCircle.width; height: width;
                    anchors.centerIn: parent;

                    Label {
                        id: labelAlarm
                        objectName: "labelAlarm"

                        anchors.centerIn: parent
                        horizontalAlignment: Text.AlignHCenter
                        verticalAlignment: Text.AlignVCenter
                        color: Theme.palette.normal.baseText

                        fontSize: "x-large"
                        text: labelAlarm.text = Utils.zeroleft(addAlarmFace.hours, 2) + ":" + Utils.zeroleft(addAlarmFace.minutes, 2)
                    }
                }
            }

            // Element to set the name of alarm
            TextField {
                id: nameAlarm

                text: alarm.message;
                anchors.horizontalCenter: addAlarmFace.horizontalCenter
                width: addAlarmFace.width/2
                style: TextFieldStyle {
                    background: Item {}
                }
                // #FIXME: The anchoring needs to be done properly
                primaryItem: Image {
                    fillMode: Image.PreserveAspectFit
                    source: Qt.resolvedUrl("../images/edit_icon.png")
                }
                color: Theme.palette.normal.baseText
                visible: true
                hasClearButton: false
                horizontalAlignment: TextInput.AlignHCenter
                placeholderText: i18n.tr("Alarm Name")
                font.pixelSize: FontUtils.sizeToPixels("large")

                InverseMouseArea {
                    visible: nameAlarm.activeFocus
                    anchors.fill: parent
                    onClicked: {
                        mainColumn.forceActiveFocus()
                        mouse.accepted = false
                    }
                }

                Image {
                    id: inputDotImage
                    visible:nameAlarm.focus
                    source: Qt.resolvedUrl("../images/dot.svg")
                    fillMode: Image.Tile
                    anchors { top: nameAlarm.bottom; left: nameAlarm.left; right: nameAlarm.right }
                }
            }

            ListItem.ThinDivider { }

            Column {
                id: alarmOptions

                anchors { left: parent.left; leftMargin: units.gu(2); right: parent.right; rightMargin: units.gu(2) }
                spacing: units.gu(1)

                OptionSelector {
                    id: occurSelector
                    text: i18n.tr("Occurs")
                    visible: repeatSelector.selectedIndex == 0
                    model: [Qt.locale().standaloneDayName(0), Qt.locale().standaloneDayName(1), Qt.locale().standaloneDayName(2), Qt.locale().standaloneDayName(3), Qt.locale().standaloneDayName(4), Qt.locale().standaloneDayName(5), Qt.locale().standaloneDayName(6)]
                    expanded: false

                    selectedIndex: alarm.date.getDay();

                    onSelectedIndexChanged: {
                        switch (selectedIndex) {
                        case 0:
                            alarm.daysOfWeek = Alarm.Sunday;
                            break;
                        case 1:
                            alarm.daysOfWeek = Alarm.Monday;
                            break;
                        case 2:
                            alarm.daysOfWeek = Alarm.Tuesday;
                            break;
                        case 3:
                            alarm.daysOfWeek = Alarm.Wednesday;
                            break;
                        case 4:
                            alarm.daysOfWeek = Alarm.Thursday;
                            break;
                        case 5:
                            alarm.daysOfWeek = Alarm.Friday;
                            break;
                        case 6:
                            alarm.daysOfWeek = Alarm.Saturday;
                            break;
                        }
                    }
                }

                ListItem.MultiValue {
                    id: multipleOccurSelector
                    anchors.left: parent.left; anchors.leftMargin: -units.gu(2)
                    text: i18n.tr("Occurence")
                    values: getValues()
                    visible: repeatSelector.selectedIndex == 2
                    onClicked: PopupUtils.open(Qt.resolvedUrl("AlarmDays.qml"), multipleOccurSelector, {"alarm": alarm});

                    function getValues() {
                        var occurs = [];
                        if (alarm.daysOfWeek & Alarm.Monday) occurs.push(Qt.locale().standaloneDayName(1));
                        if (alarm.daysOfWeek & Alarm.Tuesday) occurs.push(Qt.locale().standaloneDayName(2));
                        if (alarm.daysOfWeek & Alarm.Wednesday) occurs.push(Qt.locale().standaloneDayName(3));
                        if (alarm.daysOfWeek & Alarm.Thursday) occurs.push(Qt.locale().standaloneDayName(4));
                        if (alarm.daysOfWeek & Alarm.Friday) occurs.push(Qt.locale().standaloneDayName(5));
                        if (alarm.daysOfWeek & Alarm.Saturday) occurs.push(Qt.locale().standaloneDayName(6));
                        if (alarm.daysOfWeek & Alarm.Sunday) occurs.push(Qt.locale().standaloneDayName(7));
                        return occurs;
                    }
                }

                OptionSelector {
                    id: repeatSelector
                    text: i18n.tr("Repeats")
                    model: [i18n.tr("One Time"), i18n.tr("Daily"), i18n.tr("Weekly")]
                    expanded: false

                    selectedIndex: {
                        if (alarm.type == Alarm.OneTime)
                            return 0;
                        else if (alarm.type == Alarm.Repeating) {
                            if (alarm.daysOfWeek === Alarm.Daily)
                                return 1;
                            else
                                return 2;
                        }
                    }

                    onSelectedIndexChanged: {
                        switch (selectedIndex) {
                        case 0:
                            alarm.type = Alarm.OneTime;
                            break;
                        case 1:
                            alarm.type = Alarm.Repeating;
                            alarm.daysOfWeek = Alarm.Daily;
                            break;
                        case 2:
                            alarm.type = Alarm.Repeating;
                            break;
                        }
                    }
                }
            }
        }
    }

    // Toolbar to show back and save alarm button
    tools: ToolbarItems {
        id: toolbarAddAlarm

        opened: true
        locked: true

        ToolbarButton {
            id: addPresetToolbarButton
            objectName: "saveAlarmButton"
            iconSource: Qt.resolvedUrl("../images/select_icon.png")
            text: i18n.tr("Save")
            onTriggered: {
                if (addAlarmPage.isNewAlarm)
                    saveNewAlarm()
                else
                    updateAlarm()
            }
        }
    }
}

